/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.data.database.repository

import net.moonleay.lilJudd.data.database.entry.MatchPlanningDataData
import net.moonleay.lilJudd.data.database.tables.MatchPlanningData
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction

object MatchPlanningDataRepository {

    fun getAll(): List<MatchPlanningDataData> {
        val dataList = mutableListOf<MatchPlanningDataData>()
        transaction {
            MatchPlanningData.selectAll().forEach {
                dataList.add(
                    MatchPlanningDataData(
                        it[MatchPlanningData.id],
                        it[MatchPlanningData.serverid],
                        it[MatchPlanningData.channelid],
                        it[MatchPlanningData.matchtype],
                        it[MatchPlanningData.registererid],
                        it[MatchPlanningData.roleid],
                        it[MatchPlanningData.opponentName],
                        it[MatchPlanningData.messageid],
                        it[MatchPlanningData.timestamp],
                        it[MatchPlanningData.jobstr]
                    )
                )
            }
        }
        return dataList
    }

    fun get(id: Int): MatchPlanningDataData? =
        transaction {
            MatchPlanningData.selectAll().where { MatchPlanningData.id eq id }.firstOrNull()?.let {
                MatchPlanningDataData(
                    it[MatchPlanningData.id],
                    it[MatchPlanningData.serverid],
                    it[MatchPlanningData.channelid],
                    it[MatchPlanningData.matchtype],
                    it[MatchPlanningData.registererid],
                    it[MatchPlanningData.roleid],
                    it[MatchPlanningData.opponentName],
                    it[MatchPlanningData.messageid],
                    it[MatchPlanningData.timestamp],
                    it[MatchPlanningData.jobstr]
                )
            }
        }

    fun getFromMessageInChannelInServer(messageID: Long, channelID: Long, serverID: Long): MatchPlanningDataData? =
        transaction {
            MatchPlanningData.selectAll().where {
                MatchPlanningData.messageid eq (messageID) and (
                        MatchPlanningData.serverid eq (serverID)) and (
                        MatchPlanningData.channelid eq (channelID))
            }.firstOrNull()?.let {
                MatchPlanningDataData(
                    it[MatchPlanningData.id],
                    it[MatchPlanningData.serverid],
                    it[MatchPlanningData.channelid],
                    it[MatchPlanningData.matchtype],
                    it[MatchPlanningData.registererid],
                    it[MatchPlanningData.roleid],
                    it[MatchPlanningData.opponentName],
                    it[MatchPlanningData.messageid],
                    it[MatchPlanningData.timestamp],
                    it[MatchPlanningData.jobstr]
                )
            }
        }

    fun delete(id: Int) {
        transaction {
            MatchPlanningData.deleteWhere { MatchPlanningData.id eq id }
        }
    }

    fun write(data: MatchPlanningDataData): Int =
        transaction {
            MatchPlanningData.insert {
                it[serverid] = data.serverID
                it[channelid] = data.channelID
                it[matchtype] = data.matchType
                it[registererid] = data.registererID
                it[roleid] = data.roleID
                it[opponentName] = data.opponentName
                it[messageid] = data.messageID
                it[timestamp] = data.timestamp
                it[jobstr] = data.jobString
            } get MatchPlanningData.id
        }
}
