/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.data.database.repository

import net.moonleay.lilJudd.data.database.entry.PlanningNotifierRolesData
import net.moonleay.lilJudd.data.database.tables.PlanningNotifierRoles
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction

object PlanningNotifierRolesRepository {

    fun getAll(): List<PlanningNotifierRolesData> {
        val dataList = mutableListOf<PlanningNotifierRolesData>()
        transaction {
            for (pnr in PlanningNotifierRoles.selectAll()) {
                dataList.add(
                    PlanningNotifierRolesData(
                        pnr[PlanningNotifierRoles.id],
                        pnr[PlanningNotifierRoles.serverid],
                        pnr[PlanningNotifierRoles.channelid],
                        pnr[PlanningNotifierRoles.hastimeroleid],
                        pnr[PlanningNotifierRoles.wantstobenotifiedid]
                    )
                )
            }
        }
        return dataList
    }

    fun getForChannel(channelID: Long): PlanningNotifierRolesData? =
        transaction {
            PlanningNotifierRoles.selectAll().where { PlanningNotifierRoles.channelid eq channelID }.firstOrNull()?.let {
                PlanningNotifierRolesData(
                    it[PlanningNotifierRoles.id],
                    it[PlanningNotifierRoles.serverid],
                    it[PlanningNotifierRoles.channelid],
                    it[PlanningNotifierRoles.hastimeroleid],
                    it[PlanningNotifierRoles.wantstobenotifiedid]
                )
            }
        }

    fun getForChannelInServer(channelID: Long, serverID: Long): PlanningNotifierRolesData? =
        transaction {
            PlanningNotifierRoles.selectAll()
                .where { PlanningNotifierRoles.channelid eq channelID and (PlanningNotifierRoles.serverid eq serverID) }
                .firstOrNull()?.let {
                PlanningNotifierRolesData(
                    it[PlanningNotifierRoles.id],
                    it[PlanningNotifierRoles.serverid],
                    it[PlanningNotifierRoles.channelid],
                    it[PlanningNotifierRoles.hastimeroleid],
                    it[PlanningNotifierRoles.wantstobenotifiedid]
                )
            }
        }

    fun existsInChannel(channelID: Long): Boolean =
        transaction {
            PlanningNotifierRoles.selectAll().where { PlanningNotifierRoles.channelid eq channelID }.count() > 0
        }

    fun existsInChannelFromSever(channelID: Long, serverID: Long): Boolean =
        transaction {
            PlanningNotifierRoles.selectAll()
                .where { PlanningNotifierRoles.channelid eq channelID and (PlanningNotifierRoles.serverid eq serverID) }
                .count() > 0
        }

    fun write(data: PlanningNotifierRolesData) {
        transaction {
            PlanningNotifierRoles.insert {
                it[serverid] = data.serverID
                it[channelid] = data.channelID
                it[hastimeroleid] = data.hasTimeRoleID
                it[wantstobenotifiedid] = data.wantsToBeNotifiedID
            } get PlanningNotifierRoles.id
        }
    }

    fun delete(id: Int) {
        transaction {
            PlanningNotifierRoles.deleteWhere { PlanningNotifierRoles.id eq id }
        }
    }
}
