/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.data.database.repository

import net.moonleay.lilJudd.data.database.entry.TimePlanningMessagesData
import net.moonleay.lilJudd.data.database.tables.TimePlanningMessages
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction

object TimePlanningMessagesRepository {

    fun write(data: TimePlanningMessagesData): Int =
        transaction {
            TimePlanningMessages.insert {
                it[serverid] = data.serverID
                it[channelid] = data.channelID
                it[weekstamp] = data.weekstamp
                it[messageids] = data.messageIDs
            } get TimePlanningMessages.id
        }

    fun getWeek(stamp: Long): List<TimePlanningMessagesData> {
        val dataList = mutableListOf<TimePlanningMessagesData>()
        transaction {
            for (pnr in TimePlanningMessages.selectAll().where { TimePlanningMessages.weekstamp eq (stamp) }) {
                dataList.add(
                    TimePlanningMessagesData(
                        pnr[TimePlanningMessages.id],
                        pnr[TimePlanningMessages.serverid],
                        pnr[TimePlanningMessages.channelid],
                        pnr[TimePlanningMessages.weekstamp],
                        pnr[TimePlanningMessages.messageids]
                    )
                )
            }
        }
        return dataList
    }

    fun getWeekInChannel(stamp: Long, channelID: Long): TimePlanningMessagesData? =
        transaction {
            TimePlanningMessages.selectAll()
                .where { TimePlanningMessages.weekstamp eq (stamp) and (TimePlanningMessages.channelid eq channelID) }.firstOrNull()?.let {
                TimePlanningMessagesData(
                    it[TimePlanningMessages.id],
                    it[TimePlanningMessages.serverid],
                    it[TimePlanningMessages.channelid],
                    it[TimePlanningMessages.weekstamp],
                    it[TimePlanningMessages.messageids]
                )
            }
        }


}
