/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.enumChoice
import com.kotlindiscord.kord.extensions.commands.converters.impl.string
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import dev.kord.core.behavior.createRole
import dev.kord.rest.builder.message.actionRow
import dev.kord.rest.builder.message.embed
import net.moonleay.lilJudd.data.database.entry.MatchPlanningDataData
import net.moonleay.lilJudd.data.database.repository.MatchPlanningDataRepository
import net.moonleay.lilJudd.extensions.component.MatchTypes
import net.moonleay.lilJudd.jobs.MatchJob
import net.moonleay.lilJudd.jobs.component.JobManager
import net.moonleay.lilJudd.util.EmbedColor
import net.moonleay.lilJudd.util.EmbedUtil
import net.moonleay.lilJudd.util.MessageUtil
import net.moonleay.lilJudd.util.TimeUtil

class MatchExtension : Extension() {

    override val name = "match"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        publicSlashCommand(::MatchArguments) {
            name = "match"
            description = "Create a match"
            this.action {
                val args = this.arguments
                val m = this.member!!
                val gID = this.guild!!.id.value
                val cID = this.channel.id.value
                val opponent = args.opponent
                if (!TimeUtil.validateDateString(args.timeStamp)) {
                    this.respond {
                        this.embed {
                            this.color = EmbedColor.ERROR.color
                            this.title = "400: Bad Request"
                            this.description = "The given timestamp is invalid.\n" +
                                    "Please use the format \"dd.MM.yyyy HH:mm\"."
                            this.footer {
                                this.icon = m.asUser().avatar?.cdnUrl?.toUrl()
                                this.text = MessageUtil.getFooter(m.asUser())
                            }
                        }
                    }
                    return@action
                }
                // filter time to date:
                val zdt = TimeUtil.getDateFromString(args.timeStamp)
                // get the string for the cronjob
                val jobString = TimeUtil.getCronjobStringFromDate(zdt)
                // create the role
                val role = this.guild!!.createRole {
                    this.name =
                        "${args.matchType.readableName} Vs ${opponent} At ${zdt.dayOfMonth}/${zdt.month}/${zdt.year} ${zdt.hour}:${zdt.minute}"
                    this.mentionable = true
                }
                // Check if the role was created successfully
                if (role == null) {
                    this.respond {
                        this.embed {
                            this.color = EmbedColor.ERROR.color
                            this.title = "500: Internal Error"
                            this.description = "Could not find created role.\n" +
                                    "It seems, that said role could not be created."
                            this.footer {
                                this.icon = m.asUser().avatar?.cdnUrl?.toUrl()
                                this.text = MessageUtil.getFooter(m.asUser())
                            }
                        }
                    }
                    return@action
                }
                val msg = this.respond {
                    val eb = MessageUtil.getEmbedWithTable(
                        EmbedColor.INFO,
                        args.matchType.readableName,
                        "***Vs. $opponent***\n" +
                                "At ${args.timeStamp}\n" +
                                "Registered by ${m.mention}",
                        mapOf(
                            "Signed up" to listOf(),
                            "Unavailable" to listOf(),
                        )
                    )
                    this.embed {
                        this.color = eb.color
                        this.title = eb.title
                        this.description = eb.description
                        this.fields = eb.fields
                        this.footer {
                            this.icon = m.asUser().avatar?.cdnUrl?.toUrl()
                            this.text = MessageUtil.getFooter(m.asUser())
                        }
                    }

                    this.actionRow {
                        this.components.addAll(EmbedUtil.getMatchButtons().components)
                    }
                }
                val tID = MatchPlanningDataRepository.write(
                    MatchPlanningDataData(
                        0,
                        gID.toLong(),
                        cID.toLong(),
                        args.matchType.readableName,
                        m.id.value.toLong(),
                        role.id.value.toLong(),
                        opponent,
                        msg.id.value.toLong(),
                        (zdt.toEpochSecond() * 1000),
                        jobString
                    )
                )
                if (tID == null || tID <= 0) {
                    return@action // Not saved to db
                    // TODO: Add error message
                }
                JobManager.addJob(
                    MatchJob(
                        jobString,
                        tID,
                        "${args.matchType.readableName}_Vs_${opponent}_[${tID}]-${gID}_${cID}",
                    )
                )
            }
        }
    }

    inner class MatchArguments : Arguments() {

        val matchType by enumChoice<MatchTypes> {
            this.name = "match"
            this.description = "The type of match"
            this.typeName = "en_US"
        }


        val timeStamp by string {
            this.name = "timestamp"
            this.description = "The timestamp of the match. Format \"dd.MM.yyyy HH:mm\"."
        }

        val opponent by string {
            this.name = "opponent"
            this.description = "The opponent"
        }
    }
}
