/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.extensions

import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.utils.hasPermission
import dev.kord.common.entity.Permission
import dev.kord.core.behavior.channel.createMessage
import dev.kord.rest.builder.message.actionRow
import dev.kord.rest.builder.message.embed
import kotlinx.coroutines.delay
import net.moonleay.lilJudd.Bot
import net.moonleay.lilJudd.data.database.entry.TimePlanningMessagesData
import net.moonleay.lilJudd.data.database.repository.TimePlanningMessagesRepository
import net.moonleay.lilJudd.util.*
import java.time.ZoneId
import java.time.ZonedDateTime

/* This extension has no proper use.
   It is used in testing to test stuff, without having to wait for certain events to trigger. */
class SendPlannerExtension : Extension() {
    override val name = "sendplanner"
    override val allowApplicationCommandInDMs: Boolean
        get() = false

    override suspend fun setup() {
        publicSlashCommand {
            name = "sendplanner"
            description = "Send the planner for the current week"
            this.action {
                if (!this.member!!.asMember(this.guild!!.id)
                        .hasPermission(Permission.Administrator)
                ) {
                    val res = this.respond {
                        this.embed {
                            this.color = EmbedColor.ERROR.color
                            this.title = "401: Not Authorized"
                            this.description =
                                "You need the Administrator permission to use this command."
                            this.footer {
                                this.icon = user.asUser().avatar?.cdnUrl?.toUrl()
                                this.text = MessageUtil.getFooter(user)
                            }
                        }
                    }
                    res.delete()
                    return@action
                }
                val res = this.respond {
                    this.embed {
                        this.color = EmbedColor.INFO.color
                        this.title = "200: Success"
                        this.description = "Sending the planner.\n" +
                                "This may take a while, please be patient."
                        this.footer {
                            this.icon = user.asUser().avatar?.cdnUrl?.toUrl()
                            this.text = MessageUtil.getFooter(user)
                        }
                    }
                }
                res.delete() // Delete the response
                val c = this.getChannel().asChannel()
                var msgStr = ""
                var then =
                    ZonedDateTime.now(ZoneId.of("Europe/Berlin")).withDayOfMonth(TimeUtil.getMondayDayOfMonth())
                        .withHour(4)
                        .withMinute(0).withSecond(0)
                c.createMessage {
                    this.embed {
                        this.color = EmbedColor.INFO.color
                        this.title = "Time Planning Feature"
                        this.description = "Do you have time on the following Days?"
                        this.footer {
                            this.icon = Bot.bot.kordRef.getSelf().avatar?.cdnUrl?.toUrl()
                            this.text = MessageUtil.getFooter()
                        }
                    }
                }
                delay(1000)
                repeat(7) {
                    val eb = MessageUtil.getEmbedWithTable(
                        EmbedColor.INFO,
                        "",
                        "${then.dayOfWeek.name}, ${then.dayOfMonth}.${then.monthValue}.${then.year} /${it + 1}. weekday",
                        mapOf(
                            "Is available" to listOf(),
                            "May be available" to listOf(),
                            "Is not available" to listOf()
                        )
                    )

                    val msg = c.createMessage {
                        this.embed {
                            this.color = eb.color
                            this.title = eb.title
                            this.description = eb.description
                            this.fields = eb.fields
                            this.footer {
                                this.icon = Bot.bot.kordRef.getSelf().avatar?.cdnUrl?.toUrl()
                                this.text = MessageUtil.getFooter()
                            }
                        }

                        this.actionRow {
                            this.components.addAll(EmbedUtil.getTimePlannerButtons().components)
                        }
                    }
                    msgStr += "${it}:${msg.id.value};"
                    then = then.plusDays(1).withHour(4).withMinute(0).withSecond(0)
                    Logger.out("Finished sending day $it/ This was manually triggered")
                    delay(1000)
                }

                // Save the message ids
                TimePlanningMessagesRepository.write(
                    TimePlanningMessagesData(
                        -1,
                        c.data.guildId.value?.value!!.toLong(),
                        c.id.value.toLong(),
                        (TimeUtil.getWeekStamp().toEpochSecond()),
                        msgStr
                    )
                )
                Logger.out("Finished with ${c.data.guildId.value}")
            }
        }
    }
}
