/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.extensions

import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.utils.hasPermission
import dev.kord.common.entity.Permission
import dev.kord.rest.builder.message.embed
import net.moonleay.lilJudd.features.AvailabilityManager
import net.moonleay.lilJudd.util.EmbedColor
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil

class UpdateRolesExtension : Extension() {
    override val name = "updateroles"
    override val allowApplicationCommandInDMs: Boolean
        get() = false

    override suspend fun setup() {
        publicSlashCommand {
            name = "updateroles"
            description = "Update the roles of the members in the current server"
            this.action {
                if (!this.member!!.asMember(this.guild!!.id)
                        .hasPermission(Permission.Administrator)
                ) {
                    this.respond {
                        this.embed {
                            this.color = EmbedColor.ERROR.color
                            this.title = "401: Not Authorized"
                            this.description =
                                "You need the Administrator permission to use this command."
                            this.footer {
                                this.icon = user.asUser().avatar?.cdnUrl?.toUrl()
                                this.text = MessageUtil.getFooter(user)
                            }
                        }
                    }
                    return@action
                }
                this.respond {
                    this.embed {
                        this.color = EmbedColor.INFO.color
                        this.title = "200: Success"
                        this.description = "Updating roles.\n" +
                                "This may take a while, please be patient."
                        this.footer {
                            this.icon = user.asUser().avatar?.cdnUrl?.toUrl()
                            this.text = MessageUtil.getFooter(user)
                        }
                    }
                }
                Logger.out("Starting to update roles...")

                AvailabilityManager.updateInChannel(this.channel.id)
            }
        }
    }
}
