/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.features

import dev.kord.common.entity.Snowflake
import net.moonleay.lilJudd.Bot
import net.moonleay.lilJudd.data.database.entry.MatchPlanningDataData
import net.moonleay.lilJudd.data.database.repository.MatchPlanningDataRepository
import net.moonleay.lilJudd.jobs.MatchJob
import net.moonleay.lilJudd.jobs.component.JobManager
import net.moonleay.lilJudd.util.Logger

object MatchManager {
    suspend fun update() {
        Logger.out("Updating match roles...")
        val dataList = MatchPlanningDataRepository.getAll()

        for (data in dataList) {
            Logger.out("Checking match role ${data.id}...")
            if (data.timestamp < System.currentTimeMillis()) {
                Logger.out("Match role ${data.id} is expired, removing...")
                this.removeRoleFromGuild(data.serverID, data.roleID)
                MatchPlanningDataRepository.delete(data.id)
                continue
            }
            this.registerJob(data)
        }
        Logger.out("Done. Until next time! <3   ")
    }

    private fun registerJob(data: MatchPlanningDataData) {
        JobManager.addJob(
            MatchJob(
                data.jobString,
                data.id,
                "fromdb-${data.matchType}_Vs_${data.opponentName}_[${data.id}]-${data.serverID}_${data.channelID}"
            )
        )
        Logger.out("Registered job for match ${data.id}...")
    }

    private suspend fun removeRoleFromGuild(gid: Long, rid: Long): Boolean {
        val guild = Bot.bot.kordRef.getGuildOrNull(Snowflake(gid)) ?: return false
        val role = guild.getRoleOrNull(Snowflake(rid)) ?: return false
        role.delete()
        return true
    }
}
