/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.features

import dev.inmo.krontab.buildSchedule
import dev.inmo.krontab.doInfinity
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.UserBehavior
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.entity.channel.Channel
import dev.kord.core.entity.channel.MessageChannel
import dev.kord.rest.builder.message.EmbedBuilder
import dev.kord.rest.builder.message.actionRow
import dev.kord.rest.builder.message.embed
import kotlinx.coroutines.delay
import net.moonleay.lilJudd.Bot
import net.moonleay.lilJudd.data.database.entry.TimePlanningChannelsData
import net.moonleay.lilJudd.data.database.entry.TimePlanningMessagesData
import net.moonleay.lilJudd.data.database.repository.PlanningNotifierRolesRepository
import net.moonleay.lilJudd.data.database.repository.TimePlanningChannelsRepository
import net.moonleay.lilJudd.data.database.repository.TimePlanningMessagesRepository
import net.moonleay.lilJudd.extensions.FeatureManageExtension
import net.moonleay.lilJudd.features.component.FeatureEnum
import net.moonleay.lilJudd.features.component.IFeature
import net.moonleay.lilJudd.util.EmbedColor
import net.moonleay.lilJudd.util.EmbedUtil
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil
import java.time.ZoneId
import java.time.ZonedDateTime


object TimeManager : IFeature {
    override val feat: FeatureEnum
        get() = FeatureEnum.TIMEPLANNINGFEATURE

    // Register the cronjob to run at 0:01 AM UTC every Monday
    override suspend fun registerThread() {
        Logger.out("Adding message scheduler...")
        val scheduler = buildSchedule("0 0 1 * * * 0o 1w") // 0 0 4 * * * 0o 1w // 0o is UTC
        scheduler.doInfinity {
            this.runThread()
        }
    }

    private suspend fun runThread() {
        Logger.out("Starting to notify...")

        // ChannelID -> Data
        val targetedChannels = TimePlanningChannelsRepository.getAll().associateBy { it.channelID }
        val targetedRoles = PlanningNotifierRolesRepository.getAll().associateBy { it.channelID }

        lateinit var msgStr: String

        Logger.out("${targetedChannels.count()} Channels to notify with ${targetedRoles.count()} Roles to ping!")
        for (ch2 in targetedChannels.keys) {
            val ch = Snowflake(ch2)
            if (Bot.bot.kordRef.getChannel(ch) == null)
                continue // TODO: Check if the channel is valid in another shard
            val c = Bot.bot.kordRef.getChannelOf<MessageChannel>(ch)!!
            msgStr = ""
            if (targetedRoles != null && targetedRoles.keys.contains(ch2) && targetedRoles[ch2] != null) {
                c.createMessage {
                    this.content =
                        "The weekly planning starts now <@&${Snowflake(targetedRoles[ch2]?.wantsToBeNotifiedID!!)}>"
                    this.embed {
                        this.color = EmbedColor.INFO.color
                        this.title = "Time Planning Feature"
                        this.description = "Do you have time on the following Days?"
                        this.footer {
                            this.icon = Bot.bot.kordRef.getSelf().avatar?.cdnUrl?.toUrl()
                            this.text = MessageUtil.getFooter()
                        }
                    }
                }
            } else {
                c.createMessage {
                    this.embed {
                        this.color = EmbedColor.INFO.color
                        this.title = "Time Planning Feature"
                        this.description = "Do you have time on the following Days?"
                        this.footer {
                            this.icon = Bot.bot.kordRef.getSelf().avatar?.cdnUrl?.toUrl()
                            this.text = MessageUtil.getFooter()
                        }
                    }
                }
            }
            delay(2000)
            var then = ZonedDateTime.now(ZoneId.of("Europe/Berlin")).withHour(4).withMinute(0).withSecond(0)
            repeat(7) {
                val eb = MessageUtil.getEmbedWithTable(
                    EmbedColor.INFO,
                    "",
                    "${then.dayOfWeek.name}, ${then.dayOfMonth}.${then.monthValue}.${then.year} /${it + 1}. weekday",
                    mapOf(
                        "Is available" to listOf(),
                        "May be available" to listOf(),
                        "Is not available" to listOf()
                    )
                )
                val msg = c.createMessage {
                    this.embed {
                        this.color = eb.color
                        this.title = eb.title
                        this.description = eb.description
                        this.fields = eb.fields
                        this.footer {
                            this.icon = Bot.bot.kordRef.getSelf().avatar?.cdnUrl?.toUrl()
                            this.text = MessageUtil.getFooter()
                        }
                    }

                    this.actionRow {
                        this.components.addAll(EmbedUtil.getTimePlannerButtons().components)
                    }
                }
                msgStr += "${it}:${msg.id.value};"
                then = then.plusDays(1).withHour(4).withMinute(0).withSecond(0)
                Logger.out("Finished sending day $it")

                delay(1000)
            }

            // Save the message ids
            TimePlanningMessagesRepository.write(
                TimePlanningMessagesData(
                    id = -1,
                    serverID = c.data.guildId.value?.value!!.toLong(),
                    channelID = c.data.id.value.toLong(),
                    weekstamp = then.minusDays(7).toEpochSecond(),
                    messageIDs = msgStr
                )
            )
            Logger.out("Finished with ${c.data.guildId.value}")
        }
        Logger.out("Done! Until next Monday! <3   ")
    }

    override suspend fun enable(
        u: UserBehavior,
        gID: Long,
        cID: Long,
        ch: Channel,
        args: FeatureManageExtension.FeatureManagerArgs
    ): EmbedBuilder {
        if (!TimePlanningChannelsRepository.exists(cID, gID)) {
            TimePlanningChannelsRepository.write(TimePlanningChannelsData(id = -1, serverID = gID, channelID = cID))
            return MessageUtil.getEmbed(
                EmbedColor.SUCCESS,
                "200: Success",
                "The feature was enabled in channel ${args.channel.data.name.value}",
                u.asUser().username
            )
        }
        return MessageUtil.getEmbed(
            EmbedColor.ERROR,
            "409: Conflict",
            "The feature is already enabled in this channel.",
            u.asUser().username
        )
    }

    override suspend fun disable(
        u: UserBehavior,
        gID: Long,
        cID: Long,
        ch: Channel,
        args: FeatureManageExtension.FeatureManagerArgs
    ): EmbedBuilder {
        // Check if entry exists in db
        if (TimePlanningChannelsRepository.exists(cID, gID)) {
            // delete all entrys for this channel
            TimePlanningChannelsRepository.deleteFromChannelInServer(cID, gID)
            return MessageUtil.getEmbed(
                EmbedColor.SUCCESS,
                "200: Success",
                "The feature was disabled.",
                u.asUser().username
            )
        }
        // Do nothing; not in db
        return MessageUtil.getEmbed(
            EmbedColor.ERROR,
            "409: Conflict",
            "The feature is already disabled in this channel.",
            u.asUser().username
        )
    }
}
