/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.util

import dev.kord.core.behavior.UserBehavior
import dev.kord.core.entity.Embed
import dev.kord.rest.builder.message.EmbedBuilder
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

object MessageUtil {
    private val dtf: DateTimeFormatter = DateTimeFormatter.ofPattern("dd/MM/yyyy @ HH:mm:ss")

    suspend fun getFooter(u: UserBehavior? = null): String {
        val now: LocalDateTime = LocalDateTime.now()
        return ">" + dtf.format(now) + " - ${u?.asUser()?.username ?: "Automated Message"}"
    }

    ///Get a cloned embedded message
    fun getAClonedEmbed(e: Embed): EmbedBuilder {
        val ebb = EmbedBuilder()
        ebb.color = e.color
        ebb.title = e.title
        e.fields.forEach {
            val fb = EmbedBuilder.Field()
            fb.name = it.name
            fb.value = it.value
            fb.inline = it.inline
            ebb.fields.add(fb)
        }
        ebb.description = e.description
        return ebb
    }

    fun getAClonedEmbed(e: EmbedBuilder): EmbedBuilder {
        val ebb = EmbedBuilder()
        ebb.color = e.color
        ebb.title = e.title
        e.fields.forEach {
            val fb = EmbedBuilder.Field()
            fb.name = it.name
            fb.value = it.value
            fb.inline = it.inline
            ebb.fields.add(fb)
        }
        ebb.description = e.description
        return ebb
    }

    fun getEmbedWithTable(
        color: EmbedColor,
        title: String,
        description: String,
        values: Map<String, List<String>>?
    ): EmbedBuilder {
        val ebb = getEmbedSmall(color, title, description)
        if (values != null)
            for (key in values.keys) {
                val fb = EmbedBuilder.Field()
                fb.name = key
                var s = ""
                for (value in values[key]!!)
                    s += "$value\n"
                fb.value = s
                fb.inline = true
                ebb.fields.add(fb)
            }
        return ebb
    }

    ///Get an embedded msg with title and description
    fun getEmbedSmall(
        color: EmbedColor,
        title: String,
        description: String
    ): EmbedBuilder {
        val ebb = EmbedBuilder()
        ebb.title = title
        ebb.description = description
        ebb.color = color.color
        return ebb
    }

    ///Get an embedded msg with title, description and a src
    fun getEmbed(
        color: EmbedColor,
        title: String,
        description: String,
        source: String
    ): EmbedBuilder {
        val ebb = getEmbedSmall(color, title, description)
        val now: LocalDateTime = LocalDateTime.now()
        ebb.footer = EmbedBuilder.Footer()
        ebb.footer!!.text = ">" + dtf.format(now) + " - $source"
        return ebb
    }

}
